"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/ui/form/__init__.py $
$Id: __init__.py 23198 2003-11-19 20:17:27Z rmasse $
"""
from dulcinea.ui.form import string_list_widget, address_widget, user_widget, \
     date_widget
