'''$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/__init__.py $
$Id: __init__.py 22753 2003-10-15 21:26:17Z nascheme $
'''

from dulcinea.site_util import LazyModule as _LazyModule

# import these from here but do not access their attributes at the
# module-level.  That will lead to circular import problems.
local = _LazyModule('local')
local_ui = _LazyModule('local_ui')
